package com.supervision.nxllmcommon.util;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;

import java.util.Date;
import java.util.Map;
import java.util.Objects;
import java.util.function.Supplier;

@Slf4j
public class RedisSequenceUtil {

    /**
     * redisTemplate:不要直接使用redisTemplate属性，强烈建议通过getRedisTemplate()方法获取对象
     */
  private volatile static RedisTemplate<String, String> redisTemplate = null;


    /**
     * 获取流程编号
     * @return 流程编号规则 a000001-a999999 b000001-b999999 ...
     */
    public static String getProcessNo(){
        // 从零开始计算
        long processNoSeq = getIncrement("process_no_seq") -1;

        char prefix = (char) ('a' + processNoSeq / 1000000);
        Long suffix = processNoSeq % 1000000;
        return prefix + String.format("%06d", suffix);

    }

    /**
     * 获取复合疾病编号。生成规则：当天日期（YYYMMDD）+自增序列号（3位）20240104001
     * @return
     */
   public static String getComplexDiseaseNo(){
       String nowDay = DateUtil.format(new Date(), "YYMMdd");
       return "mp"+nowDay + String.format("%03d", getIncrement("complex_disease_no_seq_"+nowDay));
   }

    /**
     * Q0000001
     * @param supplier 初始值
     * @return Q0000001
     */
   public static String getQuestionLibraryCode(Supplier<Long> supplier){

       RedisTemplate<String,String> redisTemplate = getRedisTemplate();
       if (Objects.nonNull(supplier) &&
               Boolean.FALSE.equals(redisTemplate.hasKey("question_library_code_seq"))){
           Long code = supplier.get();
           if (Objects.nonNull(code)){
               redisTemplate.opsForValue().set("question_library_code_seq",code.toString());
           }
       }
       Long increment = getIncrement("question_library_code_seq");

       return "Q" + String.format("%07d", increment);
   }

    /**
     * A0000029
     * @param supplier 初始值
     * @return A0000029
     */
    public static String getQuestionLibraryDefaultAnswerCode(Supplier<Long> supplier){

        RedisTemplate<String,String> redisTemplate = getRedisTemplate();
        if (Objects.nonNull(supplier) &&
                Boolean.FALSE.equals(redisTemplate.hasKey("question_library_default_answer_code_seq"))){
            Long code = supplier.get();
            if (Objects.nonNull(code)){
                redisTemplate.opsForValue().set("question_library_default_answer_code_seq",code.toString());
            }
        }
        Long increment = getIncrement("question_library_default_answer_code_seq");

        return "A" + String.format("%07d", increment);
    }

    /**
     * 获取自增长值，增长步伐为1
     * @param key redis key
     * @return 自增序列号
     */
    public static Long getIncrement(String key){
        ValueOperations<String, String> operations = getRedisTemplate().opsForValue();
        return operations.increment( key,1L);
    }

    /**
     * 获取redisTemplate
     * @return redisTemplate
     */
    public static RedisTemplate getRedisTemplate() {
        if (redisTemplate == null) {
            synchronized (RedisSequenceUtil.class){
                if (null == redisTemplate){
                    redisTemplate = getRedisTemplateInContext();
                }
            }
        }
        return redisTemplate;
    }

    private static RedisTemplate<String, String> getRedisTemplateInContext() {
        ApplicationContext applicationContext = SpringBeanUtil.getApplicationContext();
        if (null == applicationContext){
            return null;
        }
        // 优先选择stringRedisTemplate
        if (applicationContext.containsBean("stringRedisTemplate")){
            return applicationContext.getBean("stringRedisTemplate",StringRedisTemplate.class);
        }
        Map<String, RedisTemplate> redisTemplateMap = applicationContext.getBeansOfType(RedisTemplate.class);
        if (CollUtil.isNotEmpty(redisTemplateMap)){
            return CollUtil.getFirst(redisTemplateMap.values());
        }
        return null;
    }

}
